/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSImageCell;

import dstep.appkit.NSCell;
import dstep.appkit.NSImage;
import dstep.foundation.NSCoder;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSUInteger NSImageAlignment;
alias NSUInteger NSImageFrameStyle;

enum
{
	NSImageAlignCenter = 0,
	NSImageAlignTop,
	NSImageAlignTopLeft,
	NSImageAlignTopRight,
	NSImageAlignLeft,
	NSImageAlignBottom,
	NSImageAlignBottomLeft,
	NSImageAlignBottomRight,
	NSImageAlignRight
}

enum
{
	NSImageFrameNone = 0,
	NSImageFramePhoto,
	NSImageFrameGrayBezel,
	NSImageFrameGroove,
	NSImageFrameButton
}

class NSImageCell : NSCell, INSCopying, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}

	uint imageAlignment ()
	{
		return invokeObjcSelf!(uint, "imageAlignment");
	}

	void setImageAlignment (uint newAlign)
	{
		return invokeObjcSelf!(void, "setImageAlignment:", uint)(newAlign);
	}

	uint imageScaling ()
	{
		return invokeObjcSelf!(uint, "imageScaling");
	}

	void setImageScaling (uint newScaling)
	{
		return invokeObjcSelf!(void, "setImageScaling:", uint)(newScaling);
	}

	uint imageFrameStyle ()
	{
		return invokeObjcSelf!(uint, "imageFrameStyle");
	}

	void setImageFrameStyle (uint newStyle)
	{
		return invokeObjcSelf!(void, "setImageFrameStyle:", uint)(newStyle);
	}

}

