/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSPathControl;

import dstep.appkit.NSColor;
import dstep.appkit.NSControl;
import dstep.appkit.NSDragging;
import dstep.appkit.NSMenu;
import dstep.appkit.NSOpenPanel;
import dstep.appkit.NSPasteboard;
import dstep.appkit.NSPathCell;
import dstep.appkit.NSPathComponentCell;
import dstep.foundation.NSArray;
import dstep.foundation.NSObject;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSPathControl : NSControl
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	NSURL URL ()
	{
		return invokeObjcSelf!(NSURL, "URL");
	}

	void setURL (NSURL url)
	{
		return invokeObjcSelf!(void, "setURL:", NSURL)(url);
	}

	SEL doubleAction ()
	{
		return invokeObjcSelf!(SEL, "doubleAction");
	}

	void setDoubleAction (SEL action)
	{
		return invokeObjcSelf!(void, "setDoubleAction:", SEL)(action);
	}

	int pathStyle ()
	{
		return invokeObjcSelf!(int, "pathStyle");
	}

	void setPathStyle (int style)
	{
		return invokeObjcSelf!(void, "setPathStyle:", int)(style);
	}

	NSPathComponentCell clickedPathComponentCell ()
	{
		return invokeObjcSelf!(NSPathComponentCell, "clickedPathComponentCell");
	}

	NSArray pathComponentCells ()
	{
		return invokeObjcSelf!(NSArray, "pathComponentCells");
	}

	void setPathComponentCells (NSArray cells)
	{
		return invokeObjcSelf!(void, "setPathComponentCells:", NSArray)(cells);
	}

	void setBackgroundColor (NSColor color)
	{
		return invokeObjcSelf!(void, "setBackgroundColor:", NSColor)(color);
	}

	NSColor backgroundColor ()
	{
		return invokeObjcSelf!(NSColor, "backgroundColor");
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void setDelegate (Object delegate_)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(delegate_);
	}

	void setDraggingSourceOperationMask (uint mask, bool isLocal)
	{
		return invokeObjcSelf!(void, "setDraggingSourceOperationMask:forLocal:", uint, bool)(mask, isLocal);
	}
}

interface INSPathControlDelegate : INSObject
{
	bool pathControl (NSPathControl pathControl, NSPathComponentCell pathComponentCell, NSPasteboard pasteboard);
	uint pathControl (NSPathControl pathControl, INSDraggingInfo info);
	bool pathControl (NSPathControl pathControl, INSDraggingInfo info);
	void pathControl (NSPathControl pathControl, NSOpenPanel openPanel);
	void pathControl (NSPathControl pathControl, NSMenu menu);
}

