/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSHTTPCookie;

import dstep.foundation.NSArray;
import dstep.foundation.NSDate;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;



import bindings = dstep.foundation.NSHTTPCookie_bindings;

private
{
	NSString NSHTTPCookieName_;
	NSString NSHTTPCookieValue_;
	NSString NSHTTPCookieOriginURL_;
	NSString NSHTTPCookieVersion_;
	NSString NSHTTPCookieDomain_;
	NSString NSHTTPCookiePath_;
	NSString NSHTTPCookieSecure_;
	NSString NSHTTPCookieExpires_;
	NSString NSHTTPCookieComment_;
	NSString NSHTTPCookieCommentURL_;
	NSString NSHTTPCookieDiscard_;
	NSString NSHTTPCookieMaximumAge_;
	NSString NSHTTPCookiePort_;
}

NSString NSHTTPCookieName ()
{
	if (NSHTTPCookieName_)
		return NSHTTPCookieName_;
		
	return NSHTTPCookieName_ = new NSString(bindings.NSHTTPCookieName);
}

NSString NSHTTPCookieValue ()
{
	if (NSHTTPCookieValue_)
		return NSHTTPCookieValue_;
		
	return NSHTTPCookieValue_ = new NSString(bindings.NSHTTPCookieValue);
}

NSString NSHTTPCookieOriginURL ()
{
	if (NSHTTPCookieOriginURL_)
		return NSHTTPCookieOriginURL_;
		
	return NSHTTPCookieOriginURL_ = new NSString(bindings.NSHTTPCookieOriginURL);
}

NSString NSHTTPCookieVersion ()
{
	if (NSHTTPCookieVersion_)
		return NSHTTPCookieVersion_;
		
	return NSHTTPCookieVersion_ = new NSString(bindings.NSHTTPCookieVersion);
}

NSString NSHTTPCookieDomain ()
{
	if (NSHTTPCookieDomain_)
		return NSHTTPCookieDomain_;
		
	return NSHTTPCookieDomain_ = new NSString(bindings.NSHTTPCookieDomain);
}

NSString NSHTTPCookiePath ()
{
	if (NSHTTPCookiePath_)
		return NSHTTPCookiePath_;
		
	return NSHTTPCookiePath_ = new NSString(bindings.NSHTTPCookiePath);
}

NSString NSHTTPCookieSecure ()
{
	if (NSHTTPCookieSecure_)
		return NSHTTPCookieSecure_;
		
	return NSHTTPCookieSecure_ = new NSString(bindings.NSHTTPCookieSecure);
}

NSString NSHTTPCookieExpires ()
{
	if (NSHTTPCookieExpires_)
		return NSHTTPCookieExpires_;
		
	return NSHTTPCookieExpires_ = new NSString(bindings.NSHTTPCookieExpires);
}

NSString NSHTTPCookieComment ()
{
	if (NSHTTPCookieComment_)
		return NSHTTPCookieComment_;
		
	return NSHTTPCookieComment_ = new NSString(bindings.NSHTTPCookieComment);
}

NSString NSHTTPCookieCommentURL ()
{
	if (NSHTTPCookieCommentURL_)
		return NSHTTPCookieCommentURL_;
		
	return NSHTTPCookieCommentURL_ = new NSString(bindings.NSHTTPCookieCommentURL);
}

NSString NSHTTPCookieDiscard ()
{
	if (NSHTTPCookieDiscard_)
		return NSHTTPCookieDiscard_;
		
	return NSHTTPCookieDiscard_ = new NSString(bindings.NSHTTPCookieDiscard);
}

NSString NSHTTPCookieMaximumAge ()
{
	if (NSHTTPCookieMaximumAge_)
		return NSHTTPCookieMaximumAge_;
		
	return NSHTTPCookieMaximumAge_ = new NSString(bindings.NSHTTPCookieMaximumAge);
}

NSString NSHTTPCookiePort ()
{
	if (NSHTTPCookiePort_)
		return NSHTTPCookiePort_;
		
	return NSHTTPCookiePort_ = new NSString(bindings.NSHTTPCookiePort);
}

class NSHTTPCookie : NSObject
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithProperties (NSDictionary properties)
	{
		return invokeObjcSelf!(Object, "initWithProperties:", NSDictionary)(properties);
	}

	this (NSDictionary properties)
	{
		typeof(this).alloc.initWithProperties(properties);
	}

	static Object cookieWithProperties (NSDictionary properties)
	{
		return invokeObjcSuperClass!(Object, "cookieWithProperties:", NSDictionary)(properties);
	}

	static NSDictionary requestHeaderFieldsWithCookies (NSArray cookies)
	{
		return invokeObjcSuperClass!(NSDictionary, "requestHeaderFieldsWithCookies:", NSArray)(cookies);
	}

	static NSArray cookiesWithResponseHeaderFields (NSDictionary headerFields, NSURL URL)
	{
		return invokeObjcSuperClass!(NSArray, "cookiesWithResponseHeaderFields:forURL:", NSDictionary, NSURL)(headerFields, URL);
	}

	NSDictionary properties ()
	{
		return invokeObjcSelf!(NSDictionary, "properties");
	}

	NSUInteger version_ ()
	{
		return invokeObjcSelf!(NSUInteger, "version");
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	NSString value ()
	{
		return invokeObjcSelf!(NSString, "value");
	}

	NSDate expiresDate ()
	{
		return invokeObjcSelf!(NSDate, "expiresDate");
	}

	bool isSessionOnly ()
	{
		return invokeObjcSelf!(bool, "isSessionOnly");
	}

	NSString domain ()
	{
		return invokeObjcSelf!(NSString, "domain");
	}

	NSString path ()
	{
		return invokeObjcSelf!(NSString, "path");
	}

	bool isSecure ()
	{
		return invokeObjcSelf!(bool, "isSecure");
	}

	NSString comment ()
	{
		return invokeObjcSelf!(NSString, "comment");
	}

	NSURL commentURL ()
	{
		return invokeObjcSelf!(NSURL, "commentURL");
	}

	NSArray portList ()
	{
		return invokeObjcSelf!(NSArray, "portList");
	}
}

