/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSXMLElement;

import dstep.foundation.NSArray;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSError;
import dstep.foundation.NSEnumerator;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSString;
import dstep.foundation.NSXMLNode;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSXMLElement : NSXMLNode
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithName (NSString name)
	{
		return invokeObjcSelf!(Object, "initWithName:", NSString)(name);
	}

	this (NSString name)
	{
		typeof(this).alloc.initWithName(name);
	}

	Object initWithName_URL (NSString name, NSString URI)
	{
		return invokeObjcSelf!(Object, "initWithName:URI:", NSString, NSString)(name, URI);
	}

	Object initWithName (NSString name, NSString string)
	{
		return invokeObjcSelf!(Object, "initWithName:stringValue:", NSString, NSString)(name, string);
	}

	this (NSString name, NSString string)
	{
		typeof(this).alloc.initWithName(name, string);
	}

	Object initWithXMLString (NSString string, ref NSError error)
	{
		id err = error ? new objc_object : null;
		Object result = invokeObjcSelf!(Object, "initWithXMLString:error:", NSString, id*)(string, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	this (NSString string, ref NSError error)
	{
		typeof(this).alloc.initWithXMLString(string, error);
	}

	NSArray elementsForName (NSString name)
	{
		return invokeObjcSelf!(NSArray, "elementsForName:", NSString)(name);
	}

	NSArray elementsForLocalName (NSString localName, NSString URI)
	{
		return invokeObjcSelf!(NSArray, "elementsForLocalName:URI:", NSString, NSString)(localName, URI);
	}

	void addAttribute (NSXMLNode attribute)
	{
		return invokeObjcSelf!(void, "addAttribute:", NSXMLNode)(attribute);
	}

	void removeAttributeForName (NSString name)
	{
		return invokeObjcSelf!(void, "removeAttributeForName:", NSString)(name);
	}

	void setAttributes (NSArray attributes)
	{
		return invokeObjcSelf!(void, "setAttributes:", NSArray)(attributes);
	}

	void setAttributesAsDictionary (NSDictionary attributes)
	{
		return invokeObjcSelf!(void, "setAttributesAsDictionary:", NSDictionary)(attributes);
	}

	NSArray attributes ()
	{
		return invokeObjcSelf!(NSArray, "attributes");
	}

	NSXMLNode attributeForName (NSString name)
	{
		return invokeObjcSelf!(NSXMLNode, "attributeForName:", NSString)(name);
	}

	NSXMLNode attributeForLocalName (NSString localName, NSString URI)
	{
		return invokeObjcSelf!(NSXMLNode, "attributeForLocalName:URI:", NSString, NSString)(localName, URI);
	}

	void addNamespace (NSXMLNode aNamespace)
	{
		return invokeObjcSelf!(void, "addNamespace:", NSXMLNode)(aNamespace);
	}

	void removeNamespaceForPrefix (NSString name)
	{
		return invokeObjcSelf!(void, "removeNamespaceForPrefix:", NSString)(name);
	}

	void setNamespaces (NSArray namespaces)
	{
		return invokeObjcSelf!(void, "setNamespaces:", NSArray)(namespaces);
	}

	NSArray namespaces ()
	{
		return invokeObjcSelf!(NSArray, "namespaces");
	}

	NSXMLNode namespaceForPrefix (NSString name)
	{
		return invokeObjcSelf!(NSXMLNode, "namespaceForPrefix:", NSString)(name);
	}

	NSXMLNode resolveNamespaceForName (NSString name)
	{
		return invokeObjcSelf!(NSXMLNode, "resolveNamespaceForName:", NSString)(name);
	}

	NSString resolvePrefixForNamespaceURI (NSString namespaceURI)
	{
		return invokeObjcSelf!(NSString, "resolvePrefixForNamespaceURI:", NSString)(namespaceURI);
	}

	void insertChild (NSXMLNode child, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChild:atIndex:", NSXMLNode, NSUInteger)(child, index);
	}

	void insertChildren (NSArray children, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChildren:atIndex:", NSArray, NSUInteger)(children, index);
	}

	void removeChildAtIndex (NSUInteger index)
	{
		return invokeObjcSelf!(void, "removeChildAtIndex:", NSUInteger)(index);
	}

	void setChildren (NSArray children)
	{
		return invokeObjcSelf!(void, "setChildren:", NSArray)(children);
	}

	void addChild (NSXMLNode child)
	{
		return invokeObjcSelf!(void, "addChild:", NSXMLNode)(child);
	}

	void replaceChildAtIndex (NSUInteger index, NSXMLNode node)
	{
		return invokeObjcSelf!(void, "replaceChildAtIndex:withNode:", NSUInteger, NSXMLNode)(index, node);
	}

	void normalizeAdjacentTextNodesPreservingCDATA (bool preserve)
	{
		return invokeObjcSelf!(void, "normalizeAdjacentTextNodesPreservingCDATA:", bool)(preserve);
	}
}

