/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 28, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.qtkit.QTMovieView;

import dstep.cocoa.Cocoa;
import dstep.appkit.NSUserInterfaceValidation;
import dstep.foundation.NSCoder;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;
import dstep.qtkit.QTKitDefines;
import dstep.qtkit.QTMovie;

import bindings = dstep.qtkit.QTMovieView_bindings;

private
{
	NSString QTMovieViewMovieBinding_;
	NSString QTMovieViewControllerVisibleBinding_;
	NSString QTMovieViewPreservesAspectRatioBinding_;
	NSString QTMovieViewFillColorBinding_;
}

NSString QTMovieViewMovieBinding ()
{
	if (QTMovieViewMovieBinding_)
		return QTMovieViewMovieBinding_;

	return QTMovieViewMovieBinding_ = new NSString(bindings.QTMovieViewMovieBinding);
}

NSString QTMovieViewControllerVisibleBinding ()
{
	if (QTMovieViewControllerVisibleBinding_)
		return QTMovieViewControllerVisibleBinding_;

	return QTMovieViewControllerVisibleBinding_ = new NSString(bindings.QTMovieViewControllerVisibleBinding);
}

NSString QTMovieViewPreservesAspectRatioBinding ()
{
	if (QTMovieViewPreservesAspectRatioBinding_)
		return QTMovieViewPreservesAspectRatioBinding_;

	return QTMovieViewPreservesAspectRatioBinding_ = new NSString(bindings.QTMovieViewPreservesAspectRatioBinding);
}

NSString QTMovieViewFillColorBinding ()
{
	if (QTMovieViewFillColorBinding_)
		return QTMovieViewFillColorBinding_;

	return QTMovieViewFillColorBinding_ = new NSString(bindings.QTMovieViewFillColorBinding);
}

const TQTMovieViewDelegate = `

	CIImage view (QTMovieView view, CIImage image)
	{
		return invokeObjcSelf!(CIImage, "view:willDisplayImage:", QTMovieView, CIImage)(view, image);
	}

	//mixin ObjcBindMethod!(view, "view:willDisplayImage:");

`;

class QTMovieView : NSView, INSCoding, INSUserInterfaceValidations
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	bool validateUserInterfaceItem (INSValidatedUserInterfaceItem anItem)
	{
		return invokeObjcSelf!(bool, "validateUserInterfaceItem:", INSValidatedUserInterfaceItem)(anItem);
	}
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}

	QTMovieView initWithFrame (NSRect frame)
	{
		id result = invokeObjcSelf!(id, "initWithFrame:", NSRect)(frame);
		return result is this.objcObject ? this : (result !is null ? new QTMovieView(result) : null);
	}

	this (NSRect frame)
	{
		super(QTMovieView.alloc.initWithFrame(frame).objcObject);
	}

	QTMovie movie ()
	{
		return invokeObjcSelf!(QTMovie, "movie");
	}

	bool isControllerVisible ()
	{
		return invokeObjcSelf!(bool, "isControllerVisible");
	}

	bool isEditable ()
	{
		return invokeObjcSelf!(bool, "isEditable");
	}

	float controllerBarHeight ()
	{
		return invokeObjcSelf!(float, "controllerBarHeight");
	}

	bool preservesAspectRatio ()
	{
		return invokeObjcSelf!(bool, "preservesAspectRatio");
	}

	NSColor fillColor ()
	{
		return invokeObjcSelf!(NSColor, "fillColor");
	}

	NSRect movieBounds ()
	{
		return invokeObjcSelf!(NSRect, "movieBounds");
	}

	NSRect movieControllerBounds ()
	{
		return invokeObjcSelf!(NSRect, "movieControllerBounds");
	}

	void setMovie (QTMovie movie)
	{
		return invokeObjcSelf!(void, "setMovie:", QTMovie)(movie);
	}

	void setControllerVisible (bool controllerVisible)
	{
		return invokeObjcSelf!(void, "setControllerVisible:", bool)(controllerVisible);
	}

	void setPreservesAspectRatio (bool preservesAspectRatio)
	{
		return invokeObjcSelf!(void, "setPreservesAspectRatio:", bool)(preservesAspectRatio);
	}

	void setFillColor (NSColor fillColor)
	{
		return invokeObjcSelf!(void, "setFillColor:", NSColor)(fillColor);
	}

	void setEditable (bool editable)
	{
		return invokeObjcSelf!(void, "setEditable:", bool)(editable);
	}

	void setShowsResizeIndicator (bool show)
	{
		return invokeObjcSelf!(void, "setShowsResizeIndicator:", bool)(show);
	}

	void play (Object sender)
	{
		return invokeObjcSelf!(void, "play:", Object)(sender);
	}

	void pause (Object sender)
	{
		return invokeObjcSelf!(void, "pause:", Object)(sender);
	}

	void gotoBeginning (Object sender)
	{
		return invokeObjcSelf!(void, "gotoBeginning:", Object)(sender);
	}

	void gotoEnd (Object sender)
	{
		return invokeObjcSelf!(void, "gotoEnd:", Object)(sender);
	}

	void gotoNextSelectionPoint (Object sender)
	{
		return invokeObjcSelf!(void, "gotoNextSelectionPoint:", Object)(sender);
	}

	void gotoPreviousSelectionPoint (Object sender)
	{
		return invokeObjcSelf!(void, "gotoPreviousSelectionPoint:", Object)(sender);
	}

	void gotoPosterFrame (Object sender)
	{
		return invokeObjcSelf!(void, "gotoPosterFrame:", Object)(sender);
	}

	void stepForward (Object sender)
	{
		return invokeObjcSelf!(void, "stepForward:", Object)(sender);
	}

	void stepBackward (Object sender)
	{
		return invokeObjcSelf!(void, "stepBackward:", Object)(sender);
	}

	void cut (Object sender)
	{
		return invokeObjcSelf!(void, "cut:", Object)(sender);
	}

	void copy (Object sender)
	{
		return invokeObjcSelf!(void, "copy:", Object)(sender);
	}

	void paste (Object sender)
	{
		return invokeObjcSelf!(void, "paste:", Object)(sender);
	}

	void selectAll (Object sender)
	{
		return invokeObjcSelf!(void, "selectAll:", Object)(sender);
	}

	void selectNone (Object sender)
	{
		return invokeObjcSelf!(void, "selectNone:", Object)(sender);
	}

	void delete_ (Object sender)
	{
		return invokeObjcSelf!(void, "delete:", Object)(sender);
	}

	void add (Object sender)
	{
		return invokeObjcSelf!(void, "add:", Object)(sender);
	}

	void addScaled (Object sender)
	{
		return invokeObjcSelf!(void, "addScaled:", Object)(sender);
	}

	void replace (Object sender)
	{
		return invokeObjcSelf!(void, "replace:", Object)(sender);
	}

	void trim (Object sender)
	{
		return invokeObjcSelf!(void, "trim:", Object)(sender);
	}

	void setBackButtonVisible (bool state)
	{
		return invokeObjcSelf!(void, "setBackButtonVisible:", bool)(state);
	}

	void setCustomButtonVisible (bool state)
	{
		return invokeObjcSelf!(void, "setCustomButtonVisible:", bool)(state);
	}

	void setHotSpotButtonVisible (bool state)
	{
		return invokeObjcSelf!(void, "setHotSpotButtonVisible:", bool)(state);
	}

	void setStepButtonsVisible (bool state)
	{
		return invokeObjcSelf!(void, "setStepButtonsVisible:", bool)(state);
	}

	void setTranslateButtonVisible (bool state)
	{
		return invokeObjcSelf!(void, "setTranslateButtonVisible:", bool)(state);
	}

	void setVolumeButtonVisible (bool state)
	{
		return invokeObjcSelf!(void, "setVolumeButtonVisible:", bool)(state);
	}

	void setZoomButtonsVisible (bool state)
	{
		return invokeObjcSelf!(void, "setZoomButtonsVisible:", bool)(state);
	}

	bool isBackButtonVisible ()
	{
		return invokeObjcSelf!(bool, "isBackButtonVisible");
	}

	bool isCustomButtonVisible ()
	{
		return invokeObjcSelf!(bool, "isCustomButtonVisible");
	}

	bool isHotSpotButtonVisible ()
	{
		return invokeObjcSelf!(bool, "isHotSpotButtonVisible");
	}

	bool areStepButtonsVisible ()
	{
		return invokeObjcSelf!(bool, "areStepButtonsVisible");
	}

	bool isTranslateButtonVisible ()
	{
		return invokeObjcSelf!(bool, "isTranslateButtonVisible");
	}

	bool isVolumeButtonVisible ()
	{
		return invokeObjcSelf!(bool, "isVolumeButtonVisible");
	}

	bool areZoomButtonsVisible ()
	{
		return invokeObjcSelf!(bool, "areZoomButtonsVisible");
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void setDelegate (Object delegate_)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(delegate_);
	}
}

